function [Zprob,Zpred,ZprobAll,misrate]=crosspred(type,Z,X,Ystack,Gamma,Hg,dstack)

%crosspred: Bayesian Variable Selection
%        sample-based cross-validation predictive probability
%
%***********************************************************************
%
%INPUTS:
%-------
%type    0 for nominal response, 1 for ordinal response 
%Z       class label for the training dataset.
%X       training data.
%Ystack  visited latent vector Y
%        matrix with each row corresponding to a sampled Y 
%Gamma   visited models - indeces of selected variables.
%Hg      Vector of std's
%dstack  

%OUTPUTS:
%--------
%Zprob   vector probabilities of each obs in the class=1.
%Zpred   vector of class label for each obs.
%ZprobAll
%misrate misclassification rate.

%NOTES:
%------
% Do cell2mat on Ystack (Y1=(cell2mat(Ystack'))') and on dstack
%(d1=cell2mat(dstack)) BEFORE CALLING THIS FUNCTION
%
%
%Copyright (c) 2003 Naijun Sha.
%**********************************************************************


if type==0
	n=length(Z);m=size(Ystack,1);Zprob=zeros(n,1);Zpred=zeros(n,1);
	for i=1:n
		i
    		Ystack1=delsamps(Ystack',i)';
    		Xi=delsamps(X,i);
    		for j=1:m
    			Yhat=Ystack1(j,:);alpha=mean(Yhat);
        		r=str2num(char(Gamma(j,:)));
	  		Xri=Xi(:,r);Hr=diag(Hg(r).^2);Kr=Xri'*Xri+inv(Hr);
          		beta=inv(Kr)*Xri'*Yhat';
          		Zprob(i)=Zprob(i)+normcdf(alpha+X(i,r)*beta);
    		end;
	end;
	Zprob=Zprob/m;
	Zpred(find(Zprob>0.5))=1;
	misrate=sum(Zpred~=Z)/n; 
	ZprobAll=[];  

else  %type=1 ordinal response
	n=length(Z);m=size(Ystack,1);Zprob=zeros(n,1);Zpred=zeros(n,1);
	ncat=length(unique(Z)); ZprobAll=zeros(n,ncat);
	for i=1:n
		i
    		Ystack1=delsamps(Ystack',i)';
    		Xi=delsamps(X,i);
		Zprobi=zeros(1,ncat);
    		for z=1:ncat
			for j=1:m
    				Yhat=Ystack1(j,:);alpha=mean(Yhat);
        			r=str2num(char(Gamma(j,:)));
	  			Xri=Xi(:,r);Hr=diag(Hg(r).^2);Kr=Xri'*Xri+inv(Hr);
          			beta=inv(Kr)*Xri'*Yhat';
          			Zprobi(z)=Zprobi(z)+...
				(normcdf(dstack(j,z+1)-(alpha+X(i,r)*beta))- ...
				normcdf(dstack(j,z)-(alpha+X(i,r)*beta)));
    			end
		end
		Zprobi=Zprobi/m;
		[Zprob(i), Zpred(i)]=max(Zprobi);
		ZprobAll(i,:)=Zprobi;
	end;
	Zpred=Zpred-1;
	misrate=sum(Zpred~=Z)/n;   	
end	
