function [YSt,UniSt,ProbSt, GammaSt, storeAD, storeS,dparS]= ... 
metro_prob(type,Z,h,inidim,gamprec,probprec,X,Ytilde,QqrP,RqrP,IRES1P, IRES2P, ...
w,Hg,nPar,pPar,k,delta,dpar);

%metro_prob: Metropolis - Selection prior is with fast form of updating
%**********************************************************************
%INPUTS:
% type
% Z
% inidim	  maximum dimension for the fast form of updating	
% gamprec,        initial binary vector.
% probprec,	  log-relative posterior probability of gamprec
% X,              independent variables, n by p
% Ytilde,         augmented response matrix, (n+inidim) by q
% QqrP,RqrP,	  QR-matrices for gamprec
% IRES1P,IRSE2P,  binary matrices for fast form of updating
% k,delta,        hyperparameters Inverse Wishart
% w,              hyperparameter (Bernoulli priors)
% Hg,		  hyperparameter - prior on regression coefficients
% nPar,           number of iterations
% pPar,           probabitity of Addition/Deletion
% dpar
%
%OUTPUTS:
% YSt
% UniSt
% ProbSt,	  log-relative posterior probabilities
%                 of visited vectors
% GammaSt,        cell with positions of 1's in visited vectors
% storeAD,storeS, iteration numbers of accepted additions/deletions (AD)
%                 and swaps (S)
% dparS 
%
%USAGE:
% [YSt,UniSt,ProbSt, GammaSt, storeAD, storeS,dparS]= metro_prob(...
% type,Z,inidim,gamprec,probprec,X,Ytilde,QqrP,RqrP,IRES1P,IRES2P,k,delta,w,Hg,nPar,pPar,dpar);
%
%NOTES:
% Called by bvsme_prob
% Functions called: gofg_prob
% An adding move is forced with empty model
% A delete move is forced with full model
% 3 possible priors are possible: g-prior (Hg scalar), diagonal prior
% (Hg vector of std's) and non-diagonal prior (Hg full var-cov matrix).
% With full prior QRdelete/insert updating is not possible.
%
%REFERENCE:
%  
%     
%Copyright (c) 1997-2003 Marina Vannucci.
%**********************************************************************

[nOss p]=size(X);
[q]=size(Ytilde,2);
storeAD=[];
storeS=[];
GammaSt=cell(nPar,1);
ProbSt=zeros(1,nPar);
UniSt=zeros(1,nPar);
YSt=cell(nPar,1);
if type==0
	dparS=[];
else
	dparS=cell(nPar,1);
end
if ~isempty(k) Qk=k*eye(q); end

for i=1:nPar
	flag=0;
	gamnew=gamprec;
	poszeros=find(gamprec==0);
	if length(poszeros)==p			% Forced to add variable
		chi='AD';
		choice=ceil(rand(1)*p);
		gamnew(choice)=1;	
		if (size(Hg,1)==1 & size(Hg,2)==1) %------ gPrior
			
			[probnew, QqrN, RqrN,IRES1N,IRES2N] = ...
			gofg_prob(h,gamnew,inidim,2, X, Ytilde, ...
			QqrP, RqrP,[],[], choice, k,delta, w,Hg);	
	
		else
%-------------------Reset of limit size of IRES1 and IRES2		
			if size(IRES1P,2)==0
				flag=1;
				Ytilde(nOss+inidim+1,:)=zeros(1,q);
				inidim=inidim+1;
				[probnew, QqrN, RqrN,IRES1N,IRES2N] = ...
				gofg_prob(h,gamnew,inidim,1, X,Ytilde, ...
				[],[],[],[],[],k,delta,w,Hg);
			else				
				[probnew, QqrN, RqrN,IRES1N,IRES2N] = ...
				gofg_prob(h,gamnew,inidim,2, X, Ytilde, ...
				QqrP, RqrP,IRES1P,IRES2P, choice, ...
				k,delta, w,Hg);		
			end
		end
	elseif 	isempty(poszeros)              % Forced to delete variable
		chi='AD';
		choice=ceil(rand(1)*p);
		gamnew(choice)=0;
		[probnew, QqrN, RqrN,IRES1N,IRES2N] = ...
                    gofg_prob(h,gamnew,inidim,3, X, Ytilde, QqrP, RqrP, ...
		IRES1P, IRES2P,choice, k,delta,w,Hg);
		
	else			
		x=rand(1);
		if x<pPar

%-------------------Adding/Deleting
			chi='AD';
			choice=ceil(rand(1)*p);
			if gamnew(choice)==0
				gamnew(choice)=1;	
				if (size(Hg,1)==1 & size(Hg,2)==1) %-- gPrior
			
					[probnew,QqrN,RqrN,IRES1N,IRES2N] = ...
					gofg_prob(h,gamnew,inidim,2,X,Ytilde, ...
					QqrP,RqrP,[],[],choice,k, ...
					delta,w,Hg);	
				else
%-------------------Reset of limit size of IRES1 and IRES2		
					if size(IRES1P,2)==0
						flag=1;
						Ytilde(nOss+inidim+1,:)=zeros(1,q);
						inidim=inidim+1;
						[probnew,QqrN,RqrN,IRES1N,IRES2N] = ...
						gofg_prob(h,gamnew,inidim,1,X, ...
						Ytilde,[],[],[],[],[],k,delta,w,Hg);
					else				
						[probnew,QqrN,RqrN,IRES1N,IRES2N] = ...
						gofg_prob(h,gamnew,inidim,2,X, ...
						Ytilde,QqrP,RqrP,IRES1P,IRES2P,... 
						choice,k,delta,w,Hg);		
					end
				end
			else
				gamnew(choice)=0;		
				[probnew,QqrN,RqrN,IRES1N,IRES2N] = ...
                      		gofg_prob(h,gamnew,inidim,3,X,Ytilde, ...
				QqrP,RqrP,IRES1P,IRES2P,choice,k, ...
				delta,w,Hg);	
			end
		else

%-------------------Swapping
			chi='S';
			poszeros=find(gamprec==0);
			posones=find(gamprec==1);
			choice1=ceil(rand(1)*length(poszeros));
			choice2=ceil(rand(1)*length(posones));
			elem=posones(choice2);
			gamnew(elem)=0;	
			[probnew, QqrN,RqrN,IRES1N,IRES2N]=gofg_prob(...
			h,gamnew,inidim,3,X,Ytilde,QqrP,RqrP,IRES1P, ...
			IRES2P,elem,k,delta,w,Hg);
               	elem=poszeros(choice1);  
			gamnew(elem)=1;
			[probnew,QqrN,RqrN,IRES1N,IRES2N]=gofg_prob(...
			h,gamnew,inidim,2,X,Ytilde,QqrN,RqrN,IRES1N, ...
			IRES2N,elem,k,delta,w,Hg);
		end
	end	
	move=min(0, (probnew-probprec) );
	if move==0
		eval(['store',chi,'=[store',chi,' i];'])
		QqrP=QqrN;
                RqrP=RqrN;
		IRES1P=IRES1N;
		IRES2P=IRES2N;
                probprec=probnew;
                gamprec=gamnew;   
                ProbSt(i)=probnew;     
                GammaSt{i}=num2str(find(gamnew));
		UniSt(i)=length(find(gamnew));
	else if (rand(1) < exp(move))
			eval(['store',chi,'=[store',chi,' i];'])
			QqrP=QqrN;
			RqrP=RqrN;
			IRES1P=IRES1N;
			IRES2P=IRES2N;
			probprec=probnew;
			gamprec=gamnew;
			ProbSt(i)=probnew;
			GammaSt{i}=num2str(find(gamnew));		
			UniSt(i)=length(find(gamnew));
		else
			ProbSt(i)=probprec; 
			GammaSt{i}=num2str(find(gamprec));
			UniSt(i)=length(find(gamprec));
		
			if (size(Hg,1)==1 | size(Hg,2)==1) %---- diag prior
				if flag==1 
					Ytilde(nOss+inidim,:)=[];
					inidim=inidim-1; 
				else
				end
			else	
			end
		end
	end

%generate Y
	
	Y=Ytilde(1:nOss,1:q);
	I=find(gamprec);
	nonnulle=length(I);
	%---------------------------------- prior of gamma
	if (length(w)==1 | length(w)==p)
		prog=sum(log((w.^gamprec).*(1-w).^(1-gamprec)));
	elseif length(w)==2
		alpha=w(2)*w(1);	% Beta prior (mean=pw(1), w(2)=alpha+beta) 
		betapar=w(2)*(1-w(1));
		gami=sum(gamprec);
		prog=gammaln(alpha+gami)+gammaln(p+betapar-gami)-...
			gammaln(alpha+betapar+p);

	end

	if (size(Hg,1)==1 & size(Hg,2)==1)      % g-prior	
		Pr=eye(nOss)+h*ones(nOss,1)*ones(nOss,1)'+Hg*eye(nOss);
		switch q
		   case 1
			mu=zeros(nOss,1);
			if type==0
				Y=multruncn(mu,Pr,Y,Z);
			else
				dpar=postcut(dpar,Y,Z);
 				Y=multruncnord(mu,Pr,dpar,Y,Z);
			end
			meany=mean(Y);
			Ytilde=Y;
			sub=QqrP(:,nonnulle+1:nOss+inidim)'*Ytilde;			% Seber, page 498
			RSP=sub'*sub;   
			QGam=(Hg/(Hg+1))*RSP + Ytilde'*Ytilde/(Hg+1);
			Qstar=QGam-(nOss^2)*(1/(nOss+(h^-1)))*(meany'*meany);
			probprec= -(q/2)*log(det((Hg+1)*eye(nOss)+h*ones(nOss,1)*ones(nOss,1)'))-...
				(1/2)*Qstar+prog;
		   otherwise
			Y=mattrunct(delta,Pr,Qk,Y,Z);
			meany=mean(Y);
			Ytilde=cat(1,Y, zeros(inidim,q));
			sub=QqrP(:,nonnulle+1:nOss+inidim)'*Ytilde;			% Seber, page 498
			RSP=sub'*sub;  
			QGam=(Hg/(Hg+1))*RSP + Ytilde'*Ytilde/(Hg+1);
			Qstar=Qk+QGam-(nOss^2)*(1/(nOss+(h^-1)))*(meany'*meany);
			probprec= -(q/2)*log(det((Hg+1)*eye(nOss)+h*ones(nOss,1)*ones(nOss,1)'))-...
				((delta+nOss+q-1)/2)*log(det(Qstar))+prog;
		end
	else
		Xr=X(:,I);
		Hr=diag(Hg(I).^2);
		Pr=eye(nOss)+h*ones(nOss,1)*ones(nOss,1)'+Xr*Hr*Xr';
		%Krinv=inv(Xr'*Xr+inv(Hr));
		%Prinv=eye(nOss)-(1/(nOss+(h^-1)))*ones(nOss,1)*ones(nOss,1)'-Xr*Krinv*Xr';

		switch q
       		case 1
          		mu=zeros(nOss,1);
			if type==0
				Y=multruncn(mu,Pr,Y,Z);
			else
				dpar=postcut(dpar,Y,Z);
 				Y=multruncnord(mu,Pr,dpar,Y,Z);
 			end	
			meany=mean(Y);
			Ytilde=cat(1,Y, zeros(inidim,q));
			sub=QqrP(:,nonnulle+1:nOss+inidim)'*Ytilde;			% Seber, page 498
			RSP=sub'*sub;                       	% residual sum products
			Qstar=-(nOss^2)*(1/(nOss+(h^-1)))*(meany'*meany)+RSP;       % q by q
			%detH0K=det(Prinv);
			detH0K=det(Pr);
			probprec= -(q/2)*log(detH0K)-(1/2)*Qstar+prog;
			%ProbSt(i)=probprec;
		otherwise
			Y=mattrunct(delta,Pr,Qk,Y,Z);
			meany=mean(Y);
			Ytilde=cat(1,Y, zeros(inidim,q));
			sub=QqrP(:,nonnulle+1:nOss+inidim)'*Ytilde;			% Seber, page 498
			RSP=sub'*sub;                       	% residual sum products
			Qstar=Qk-(nOss^2)*(1/(nOss+(h^-1)))*(meany'*meany)+RSP;      % q by q
			detH0K=det(Pr);
			%detH0K=det(Prinv);
			probprec= -(q/2)*log(detH0K)-((delta+nOss+q-1)/2)*log(det(Qstar))+prog;
			%ProbSt(i)=probprec;
		end
	end
	YSt{i}=Y;
	if type==0
	else
		dparS{i}=dpar;
	end	
end
