function [Yfpred,I]=predico(flag,X,Y,Xf,input1,input2,input3,input4)

%predico: Bayesian Variable Selection
%         Prediction by Least Squares, Bayesian Model Averaging 
%	  
%***********************************************************************
%INPUTS:
% flag,	        - flag='LS' to perform least squares prediction
%		- flag='BMA' to  perfom Bayesian Model Averaging 
% X,Y,		training data
% Xf	        validation data
% input1,	- vector of 0/1 or cell with vars numbering
%		  (for flag='LS')
%		- matrix or cell of distinct ordered visited models
%		  (for flag='BMA')
%		- output matrix of atteso.m 
%		  (for flag='RBMA') -->NOT APPLICABLE HERE
% input2, 	- vector of posterior probability of visited models
%		  (for flag='BMA')
%		- vector with selected variables (use ricerca.m)
%		  (for flag='RBMA') --> NOT APPLICABLE
% input3,	- scalar or vector of std's or full var-cov matrix
%		  (for flag='BMA')
%		- scalar, indicates which lower percentile of non-zero 
%		  eigenvalues of X should be used
%		  (for flag='RBMA') --> NOT APPLICABLE
% input4,	- scalar, number of best models for model avaraging
%		  (for flag='BMA')
%
%OUTPUTS:
% I,		- vars numbering for flag='LS' 
%		- all variables used in prediction for flag='BMA' 
%		- same as input2 for flag='RBMA' --> NOT APPLICABLE
% Yfpred,	- prediction of Yf
%
%USAGE:
% [Yfpred,I]=predico('LS',X,Y,Xf,input1);
% [Yfpred,I]=predico('BMA',X,Y,Xf,input1,input2,input3,input4);
%
%NOTES:
% flag='BMA' works for  Hg vector of std's (diagonal 
% prior on regression coeffs)
%
%
%Copyright (c) 1997-2003 Marina Vannucci.
%**********************************************************************

if nargin==5
	gammodel=input1;
	switch class(gammodel)
		case 'cell'
			I=str2num(gammodel{1});		
		otherwise
			I=find(gammodel);
	end	
elseif nargin==8
	GammaD=input1;
	PostGamD=input2;
	Hg=input3;
	numero=input4;
end
	
[nOssP,p]=size(Xf);
[nOss]=size(X,1);
[q]=size(Y,2);

switch flag
	case 'LS'
%-------------------------------------- LS prediction with 'gamma' model
	I	
		nonnulle=length(I);
		Xfg=Xf(:,I);
		Xg=X(:,I);
		if nonnulle>nOss
			disp(' ')
			disp('Least square prediction with more variables')
			disp('than observations. Use generalized inverse')
			disp(' ')
			Beta=pinv(Xg'*Xg)*Xg'*Y;
			Yfpred=ones(nOssP,1)*mean(Y)+Xfg*Beta;
		else 
			[Qqr Rqr]=qr(Xg);
			Zeta=Qqr(:,1:nonnulle)'*Y;
			U=Qqr(:,1:nonnulle)'*Xg;
			Beta=inv(U)*Zeta;   
			Yfpred=ones(nOssP,1)*mean(Y)+Xfg*Beta;
		end		
		
	case 'BMA'
%-------------------------------------- BMA prediction with 'numero' models
    
		if (size(Hg,1)==1 & size(Hg,2)==1)
			%--------------------------- Hg scalar - gPrior 
			Ytilde=Y;

		elseif (size(Hg,1)==1 | size(Hg,2)==1)
			%--------------------------- Hg vector of std's 
			Ytilde=cat(1,Y, zeros(p,q));			 	
		end		
		I=[];
		Yfpred=zeros(nOssP,q);
		costante=sum(PostGamD(1:numero));
		for i=1:numero
			switch class(GammaD)
			case 'cell'
				Ii=str2num(GammaD{i});
			otherwise
				Ii=find(GammaD(i,:));
			end
			nonnulle=length(Ii);
			if (size(Hg,1)==1 & size(Hg,2)==1)
			%--------------------------- Hg scalar - gPrior 
				moltiplica=Hg/(Hg+1);
				Xtilde=X(:,Ii);
						
			elseif (size(Hg,1)==1 | size(Hg,2)==1)	
			%--------------------------- Hg vector of std's 
			
				moltiplica=diag(Hg(Ii));  
				XH0=X(:,Ii)*moltiplica;
				sotto=zeros(p,nonnulle);
				sotto(Ii,:)=eye(nonnulle);    
				Xtilde=cat(1, XH0, sotto);	
			end
			Xfg=Xf(:,Ii);
			Beta=moltiplica*inv(Xtilde'*Xtilde)*Xtilde'*Ytilde;
			Yfpred=Yfpred+(ones(nOssP,1)*mean(Y)+Xfg*Beta)*PostGamD(i)/costante;
			I=union(I,Ii);
		end
	end	
